@@ -3,7 +3,7 @@ package PLS::Server::Message;
 use strict;
 use warnings;
 
-use JSON::PP;
+use PLS::JSON;
 
 =head1 NAME
 
@@ -19,16 +19,17 @@ inherit from this class.
 
 =cut
 
-sub serialize {
+sub serialize
+{
     my ($self) = @_;
 
     my %content = (
-        jsonrpc => '2.0',
-        %{$self}
-    );
+                   jsonrpc => '2.0',
+                   %{$self}
+                  );
 
     my $json = encode_json \%content;
-    return $json;
-}
+    return \$json;
+} ## end sub serialize
 
 1;
